@@ -361,13 +361,13 @@ struct Rows_event_factory
 {
   int type_code;
 
-  Rows_log_event *(*create)(THD*, TABLE*, ulong, bool is_transactional);
+  Rows_log_event *(*create)(THD*, TABLE*, ulonglong, bool is_transactional);
 
   template<class RowsEventT>
   static Rows_event_factory get()
   {
     return { RowsEventT::TYPE_CODE,
-             [](THD* thd, TABLE* table, ulong flags, bool is_transactional)
+             [](THD* thd, TABLE* table, ulonglong flags, bool is_transactional)
                      -> Rows_log_event*
              {
                return new RowsEventT(thd, table, flags, is_transactional);
@@ -402,6 +402,7 @@ public:
                                        bool is_transactional);
   void set_write_error(THD *thd, bool is_transactional);
   static bool check_write_error(THD *thd);
+  static bool check_cache_error(THD *thd, binlog_cache_data *cache_data);
   int write_cache(THD *thd, IO_CACHE *cache);
   int write_cache_raw(THD *thd, IO_CACHE *cache);
   char* get_name() { return name; }
@@ -987,6 +988,7 @@ public:
   bool write_incident(THD *thd);
   void write_binlog_checkpoint_event_already_locked(const char *name, uint len);
   bool write_table_map(THD *thd, TABLE *table, bool with_annotate);
+
   void start_union_events(THD *thd, query_id_t query_id_param);
   void stop_union_events(THD *thd);
   bool is_query_in_union(THD *thd, query_id_t query_id_param);
@@ -1425,6 +1427,7 @@ static inline TC_LOG *get_tc_log_implementation()
 
 #ifdef WITH_WSREP
 IO_CACHE* wsrep_get_cache(THD *, bool);
+bool wsrep_is_binlog_cache_empty(THD *);
 void wsrep_thd_binlog_trx_reset(THD * thd);
 void wsrep_thd_binlog_stmt_rollback(THD * thd);
 #endif /* WITH_WSREP */
