@@ -109,7 +109,7 @@ int spider_udf_direct_sql_create_table_list(
       break;
   }
   if (!(direct_sql->db_names = (char**)
-    spider_bulk_malloc(spider_current_trx, 31, MYF(MY_WME | MY_ZEROFILL),
+    spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_TABLE_LIST_2, MYF(MY_WME | MY_ZEROFILL),
       &direct_sql->db_names, (uint) (sizeof(char*) * table_count),
       &direct_sql->table_names, (uint) (sizeof(char*) * table_count),
       &direct_sql->tables, (uint) (sizeof(TABLE*) * table_count),
@@ -238,7 +238,7 @@ int spider_udf_direct_sql_create_conn_key(
       + direct_sql->tgt_filedsn_length + 1
       + direct_sql->tgt_driver_length;
   if (!(direct_sql->conn_key = (char *)
-    spider_malloc(spider_current_trx, 9, direct_sql->conn_key_length + 1,
+    spider_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_KEY_1, direct_sql->conn_key_length + 1,
       MYF(MY_WME | MY_ZEROFILL)))
   )
     DBUG_RETURN(HA_ERR_OUT_OF_MEM);
@@ -389,7 +389,7 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
       spider_dbton[direct_sql->dbton_id].db_util->
         tables_on_different_db_are_joinable();
     if (!(conn = (SPIDER_CONN *)
-      spider_bulk_malloc(spider_current_trx, 32, MYF(MY_WME | MY_ZEROFILL),
+      spider_bulk_malloc(spider_current_trx, SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_1, MYF(MY_WME | MY_ZEROFILL),
         &conn, (uint) (sizeof(*conn)),
         &tmp_name, (uint) (direct_sql->conn_key_length + 1),
         &tmp_host, (uint) (direct_sql->tgt_host_length + 1),
@@ -420,7 +420,7 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
       *error_num = HA_ERR_OUT_OF_MEM;
       goto error_alloc_conn;
     }
-    conn->default_database.init_calc_mem(138);
+    conn->default_database.init_calc_mem(SPD_MID_UDF_DIRECT_SQL_CREATE_CONN_2);
 
   conn->conn_key_length = direct_sql->conn_key_length;
   conn->conn_key = tmp_name;
@@ -1437,7 +1437,7 @@ long long spider_direct_sql_body(
   DBUG_ENTER("spider_direct_sql_body");
   SPIDER_BACKUP_DASTATUS;
   if (!(direct_sql = (SPIDER_DIRECT_SQL *)
-    spider_bulk_malloc(spider_current_trx, 34, MYF(MY_WME | MY_ZEROFILL),
+    spider_bulk_malloc(spider_current_trx, SPD_MID_DIRECT_SQL_BODY_1, MYF(MY_WME | MY_ZEROFILL),
       &direct_sql, (uint) (sizeof(SPIDER_DIRECT_SQL)),
       &sql, (uint) (sizeof(char) * args->lengths[0]),
       NullS))
@@ -1672,7 +1672,7 @@ my_bool spider_direct_sql_init_body(
   if (bg)
   {
     if (!(bg_direct_sql = (SPIDER_BG_DIRECT_SQL *)
-      spider_malloc(spider_current_trx, 10, sizeof(SPIDER_BG_DIRECT_SQL),
+      spider_malloc(spider_current_trx, SPD_MID_DIRECT_SQL_INIT_BODY_1, sizeof(SPIDER_BG_DIRECT_SQL),
       MYF(MY_WME | MY_ZEROFILL)))
     ) {
       strcpy(message, "spider_bg_direct_sql() out of memory");
